# Hooks for scripts to be run at various times.

set final_setup [list];			# Scripts run after all normal init
                                        # (just prior to forking children)

set cleanup [list];			# Cleaning up, e.g. after a request
set full_cleanup [list];		# Full-scale cleanup, e.g. before fork

set atclose [list];			# Cleanup scripts put in by user tcl 

set child_init [list];                  # Run in each child at start
set child_exit [list];                  # Run in each child at exit

set server_shutdown [list];             # Script to run at Apache exit time

# A list of variables in the master interpreter which store request state,
# and should be unset when the request is over.  The list of user-tcl atclose
# scripts is the first element...

set per_req_vars [list atclose]

# Manipulating and invoking the hooks

proc run_at {when script} {
    global $when
    append $when $script "\n"
}

proc run_hooks {when} {
    global $when
    if [info exists $when] {eval [set $when]}
}

# The standard cleanup routine, used for cleaning up after handling
# a request or running a scheduled procedure, among other things:

proc aolserver_cleanup {} {
    
    # Run user-tcl cleanup scripts

    run_hooks atclose
    
    # Invoke subsystem cleanup routines...
    
    run_hooks cleanup
    
    # ... and clean up their per-request state

    global per_req_vars
    foreach g $per_req_vars { catch {global $g; unset $g} }
}

# Spawning subprocesses requires more aggressive cleanup first.

proc ns_start_subprocess {how script} {
    aolserver_cleanup
    run_hooks full_cleanup
    set script "run_hooks child_init; $script; run_hooks child_exit"
    ns_spawn_child $how $script
}

