################################################################
#
# AolServer tcl code is actually run in a slave interpreter; this allows
# us to get hooks in various things (the I/O routines in particular) in
# useful ways, and to implement AolServer functionality in tcl without
# allowing buggy adp code to mess with it.

# First, create the slave interpreter, and define C-coded ns_foo commands in it

global slaveInterp
set slaveInterp [interp create]
ns_extend_slave $slaveInterp

# Start out the slave with the fast version of ns_share.
$slaveInterp eval "set sv_ns_share_fast_p 1"

# Now, some simple utilities for invoking code within the slave:

proc return_tcl_serverror {erring_thing} {

    global slaveInterp
    set slaveError [$slaveInterp eval { set errorInfo }]

    ns_log Error "$erring_thing blew up --- backtrace follows:\n$slaveError"
    
    if {![ns_conn_inner isconnected]} {
	return
    }
    
    if {![ns_conn_inner sentheader]} {
	ns_return 500 text/plain ""
    }
    ns_write "\n\n\n$erring_thing blew up --- backtrace follows:\n$slaveError"
    ns_write $slaveError
}

proc safe_eval {cmd eval_type err_return} {

    global slaveInterp
    
    if {[catch {$slaveInterp eval $cmd} result] > 0 } {
	return_tcl_serverror $eval_type
	set result $err_return
    }

    return $result
}

