################################################################
#
# Various kinds of low-level Apache interfacing

# Dealing with configurable location args...

proc ap_config_filename {name default} {
    global ap_server_conf
    
    if [info exists ap_server_conf($name)] {
	set name $ap_server_conf($name)
    } else {
	set name $default
    }
    
    if ![string compare [file pathtype $name] relative] {
	set name [file join $ap_server_conf(SERVER_ROOT) $name]
    }

    return $name
}

proc ap_config_arg {name default} {
    global ap_server_conf

    if [info exists ap_server_conf($name)] {
	return $ap_server_conf($name)
    } else {
	return $default
    }
}

# Return variants

$slaveInterp eval {
    proc ns_returnnotfound { args }     { ns_returnstatus 404 }
    proc ns_returnunauthorized { args } { ns_returnstatus 401 }
    proc ns_returnforbidden { args }    { ns_returnstatus 403 }
    
    proc ns_returnerror { args } {
	if { [llength $args] == 3 } { set args [lrange $args 1 end] }
	if { [llength $args] != 2 } {
	    error "bad args: ns_returnerror status message"
	}
	ns_return [lindex $args 0] text/plain [lindex $args 1]
    }

    proc ns_returnbadrequest { args } {
	if { [llength $args] == 2 } { set args [lrange $args 1 end] }
	if { [llength $args] != 1 } {
	    error "bad args: ns_returnbadrequest message"
	}
	ns_return 400 text/plain [lindex $args 1]
    }

    proc ns_returnredirect { args } {
	if { [llength $args] == 2 } { set args [lrange $args 1 end] }
	
	set location [lindex $args 0]
	
	if {[string first / $location] == 0} {
	    set location [ns_conn location]$location
	}

	ns_set put [ns_conn outputheaders] "Location" $location
	ns_returnstatus 302
    }
    
    proc ns_returnfp { args } {

	if {[llength $args] == 4} { set args [lrange $args 1 end] }
	
	if {[llength $args] != 3} {
	    error "Bad args --- ns_returnfp ?conn? status type filename" 
	}

	ns_return [lindex $args 0] [lindex $args 1] ""
	ns_writefp [lindex $args 2]
    }

    proc ns_returnfile { args } {

	# Cheat:  this does *not* do if-modified-since, which
	# requires C support.  
	
	if {[llength $args] == 4} { set args [lrange $args 1 end] }
	
	if {[llength $args] != 3} {
	    error "Bad args --- ns_returnfile ?conn? status type filename" 
	}

	set filename [lindex $args 2]
	
	if ![file exists $filename] {
	    ns_returnnotfound; return
	}
	
	if [catch {open $filename} conn] {
	    ns_returnforbidden; return
	}

	fconfigure $conn -encoding binary

	ns_returnfp [lindex $args 0] [lindex $args 1] $conn

	close $conn
    }
}

# Finally, here's the code that actually handles requests...
# (NB that ADP is done in C code imported from aolserver sources).
# These call out to routines to run registered filters and procs,
# which are defined with the code that implements the rest of
# those features.

# Apache fixup handler --- runs preauth and postauth filters.

proc run_aol_filters {} {
    global filters_preauth filters_postauth
    
    set status [run_filters $filters_preauth preauth]
    if {[string compare $status filter_return]} {
	set status [run_filters $filters_postauth postauth]
    }
    if {[string compare $status filter_return]} {
	if [run_registered_procs [ns_conn_inner method] [ns_conn_inner url]] {
	    set status filter_return
	}
    }
    return $status
}

# Wrapup --- run trace handlers, and most importantly clean up
# request state

proc run_aol_wrapup {} {
    global filters_trace errorInfo
    
    if [catch {run_filters $filters_trace trace} retval] {
	# Internal error!
	puts stderr "Internal error in aolserver wrapup: $errorInfo"
    }
    
    aolserver_cleanup
    return $retval
}

# Handle requests for aolserver tcl... this could be done as a
# registered_proc (and in fact, ACS registers one...).

proc handle_aol_request {filename} {
    safe_eval [list source $filename] "tcl request" irrelevant_result
}

