################################################################
#
# Configuration frobs...
#
# Note that ns_configsections and ns_configsection are subtly
# different than the native ACS versions --- in particular, they
# you won't see ns/foo/bar as the bar entry in [ns_configsection foo].
# Fortunately, nothing in ACS appears to rely on this behavior...

global allconfig

set allconfig [ns_set create "Configuration"]

proc ns_configsections { } {
    global allconfig; return $allconfig
}

proc ns_configsection { sec } {
    global initializing
    
    set sections [ ns_configsections ]
    regsub -all \\\\ $sec / sec
    set idx [ ns_set find $sections $sec ]

    if {$idx >= 0} { return [ns_set value $sections $idx] }

    set new_set [ ns_set create $sec ]
    if $initializing {ns_set put $sections $sec $new_set}
    return $new_set
}

proc ns_config { args } {
    
    set firstarg [lindex $args 0]
    set find_arg ifind; set bool 0; set integer 0
    set usage \
	"Invalid args:  ns_config ?-exact, -bool, -int? section key ?default?"

    # Parse arguments.
    # We don't do "ns_config -exact -bool ns/server/foo/bar my_key", etc.;
    # this is bug-compatible.
    
    if {[string first - $firstarg] == 0} {
	
	if ![string compare $firstarg -exact] { set find_arg find } \
	    elseif ![string compare $firstarg -bool] { set bool 1 } \
	    elseif ![string compare $firstarg -int] { set integer 1 } \
	    else { error $usage }
	
	set args [lrange $args 1 end]
    }

    if {[llength $args] < 2 || [llength $args] > 3} then { error $usage }

    set section [lindex $args 0]; set key [lindex $args 1]; set value ""

    if {[llength $args] == 3} { set value [lindex $args 2] }

    # That's the hard part... now do the job.

    set configsec [ns_configsection $section]
    set idx [ns_set $find_arg $configsec $key]
    
    if {$idx >= 0} {set value [ns_set value $configsec $idx]}
    
    if {$integer && ![regexp {^-?[0-9]+$} $value]} {
	error "Non-integer value $value for $key in $section" 
    }

    if !$bool { return $value }

    set value [string tolower $value]
    if {[lsearch -exact {1 y yes on t true}  $value] >= 0} {return 1}
    if {[lsearch -exact {0 n no off f false} $value] >= 0} {return 0}

    error "Invalid boolean value $value for $key in $section"
}

$slaveInterp alias ns_configsections ns_configsections
$slaveInterp alias ns_configsection  ns_configsection
$slaveInterp alias ns_config         ns_config

################################################################
#
# Also defined here, functions which are used at server initialization
# time only to parse config files, and the ns_section and ns_param
# commands which are used for tcl-format nsd config files....

global preset_config_section

proc ns_section { sec } {
    global preset_config_section initializing
    if !$initializing { error "Server already configured" }
    set preset_config_section [ns_configsection $sec]
}

proc ns_param { key val } {
    global preset_config_section initializing
    
    if !$initializing { error "Server already configured" }
    
    if ![info exists preset_config_section] {
	error "You must call ns_section first"
    }
    
    ns_set put $preset_config_section $key $val
}

$slaveInterp alias ns_section        ns_section
$slaveInterp alias ns_param          ns_param

################################################################
#
# Stuff called by aol_bootstrap.tcl directly, as part of the
# process of setting up

# Called by aol_bootstrap.tcl to read .ini-format files
# (i.e., old-style aolserver config files)

proc read_ini_file { file } {
    
    set chan [ open $file ]

    while { [gets $chan line] >= 0 } {

	set line [string trimleft $line]
	
	if [regexp {^\;} $line] {
	    #comment
	    continue
	}
	
	if [regexp {^\[(.*)\]} $line dummy section] {
	    ns_section $section
	    continue
	}

	if ![string compare [string trim $line] ""] {
	    continue
	}

	set pos [string first = $line] 

	if {$pos < 0} {error "Syntax error:  line $line in $file"}

	set var [string trim [string range $line 0 [expr $pos - 1]]]
	set val [string trim [string range $line [expr $pos + 1] end]] 
	
	ns_param $var $val
    }

    close $chan
}

# Called for auxconfigdirs, tcl libraries, etc.
#
# (The corresponding aolserver code is written in C --- it's not clear
# why, since performance can't possibly be an issue...).

proc load_modules_from_dir {dir} {

    # Process the *.ini file(s) if any

    foreach file [lsort [glob -nocomplain [file join $dir *.ini]]] {
	puts stderr "Handling .ini file $file"
	read_ini_file $file
    }
    
    # Get list of tcl files in the directory
    
    set file_pattern [file join $dir *.tcl]
    set files [glob -nocomplain $file_pattern]

    # If there's an init.tcl in the list, load it in first, and clip
    # it out so we don't do that again...
    
    set init_idx [lsearch $files */init.tcl]

    if { $init_idx >= 0 } {
	set file [lindex $files $init_idx]
	puts stderr "Sourcing $file"
	safe_eval [ list source $file ] "tcl module" irrelevant_result
	set files [ lreplace $files $init_idx $init_idx ]
    }

    # Load the rest of the files in sorted order...

    foreach file [lsort $files] {
	puts stderr "Sourcing $file"
	safe_eval [ list source $file ] "tcl module" irrelevant_result
    }

    # ... and then recurse through subdirectories, if any

    set maybe_dirs [glob -nocomplain [file join $dir *]]

    foreach dir $maybe_dirs {
	if [file isdirectory $dir] {
	    load_modules_from_dir $dir
	}
    }
}

