################################################################
#
# tcl support routines for the database machinery

# Cleanups --- after each request, we need to release database handles

run_at cleanup ns_db_cleanup

# ... and before forking, we need to close them, so they aren't
#     shared with our child process.

run_at full_cleanup {    
    if {[catch {ns_db pools} poolnames] == 0} {
	foreach pool $poolnames { ns_db bouncepool $pool }
    }
}

# Lastly, getting the database pools configured.  This is invoked
# directly from aol_bootstrap.tcl after we've loaded all the parameter
# files.  We figure out the names of the configured pools here, if
# need be, since getting the aolserver C code that does it to work
# would be more trouble than this:

proc db_init_pools {} {
    set dbsec [ns_configsection ns/server/[ns_info server]/db]
    set pools ""
    set pool_idx [ns_set ifind $dbsec pools]

    if {$pool_idx > -1} {
	set pools [string trim [ns_set value $dbsec $pool_idx]]
    }

    if {![string compare $pools ""] || ![string compare $pools "*"]} {
    
	set pools ""
	set allconfig [ns_configsections]

	for {set i 0} {$i < [ns_set size $allconfig]} {incr i} {
	    set secname [ns_set key $allconfig $i]
	    if [regexp {^ns/db/pool/([^/]+)} $secname dummy poolname] {
		if {[lsearch -exact $pools $poolname] < 0} {
		    lappend pools $poolname
		}
	    }
	}

	regsub -all " " $pools , pools
	ns_set idelkey $dbsec pools
	ns_set update $dbsec pools $pools
    }

    ns_db_init;				# For the rest, invoke C setup code
}

