################################################################
#
# ns_register_filter

global filters_preauth
global filters_postauth
global filters_trace

set filters_preauth {}
set filters_postauth {}
set filters_trace {}

proc register_filter {when method url args} {

    global filters_preauth filters_postauth filters_trace initializing

    if !$initializing {
	error "Can't register filters once server is configured"
    }
    
    if {[llength args] < 1 || [llength args] > 2} {
	error "bad args: ns_register filter when method url script ?arg?"
    }
    
    set filter [list $method $url $args]
    
    if ![string compare $when preauth] {
	lappend filters_preauth $filter
    } elseif ![string compare $when postauth] {
	lappend filters_postauth $filter
    } elseif ![string compare $when trace] {
	lappend filters_trace $filter
    }
}

proc run_filter {arg reason} {
    global slaveInterp
    
    set pproc [lindex $arg 0]
    set nargs 0
    set err_what "$reason filter"

    # We catch errors in [info args] and don't process, because safe_eval
    # will detect undefined proc's farther on.  There are obvious bugs here
    # regarding variadic tcl functions, but we are bug compatible with the
    # original here.  Likewise for the differences between what's
    # implemented here and the docs...
    
    catch {set nargs [llength [$slaveInterp eval [list info args $pproc]]]}

    set parg ""
    if {[llength $arg] > 1} {set parg [lindex $arg 1]}
    
    # You know, some people might think that wiring indentation style
    # into the syntax of a language is a bad idea...

    if {$nargs == 0} {        set cmd [list $pproc]
    } elseif {$nargs == 1} { set cmd [list $pproc $reason]
    } elseif {$nargs == 2} { set cmd [list $pproc $parg $reason]
    } else { set cmd [list $pproc conn $parg $reason] }

    return [safe_eval $cmd $err_what filter_return]
}

proc run_filters {filter_list reason} {

    set method [ns_conn method]
    set url [ns_conn url]

    foreach filter $filter_list {
	if [string match [lindex $filter 0] $method] {
	    if [string match [lindex $filter 1] $url] {

		set result [run_filter [lindex $filter 2] $reason]

		if {[string compare $result filter_ok] != 0} {
		    return $result
		} 
	    }
	}
    }

    return filter_ok
}

$slaveInterp alias ns_register_filter register_filter

