################################################################
#
# ns_schedule_foo.

global ns_scheduled_scripts;		# list of {time intvl thread script}
global final_init_scripts;		# Just the scripts

set ns_scheduled_scripts ""
set final_init_scripts ""

# Running scripts... this is done in a subprocess.  

# XXX When doing config stuff, need to alter this so that it keeps
#     track of the virtual server from which the code is being run.

proc sort_by_times { a b } {
    return [ expr [lindex $a 0] - [lindex $b 0 ]]
}

proc run_scheduled_procs {} {

    global ns_scheduled_scripts
    
    while {[llength $ns_scheduled_scripts] > 0} {
	
	# Get next procedure.  The sort is a total waste of time, and
	# could easily be replaced with something much more efficient.
	# Exercise for the reader: How much would this save?  How much
	# would it cost to implement?
	
	set now [ns_time]
			    
	set ns_scheduled_scripts \
	    [lsort -command sort_by_times $ns_scheduled_scripts]
	
	set firstproc [lindex $ns_scheduled_scripts 0]
	set firstproc_time [lindex $firstproc 0]

	# Wait for the right time to arrive, if need be, then run it.
	
	if { $now < $firstproc_time } { ns_sleep [expr $firstproc_time - $now]}

	if { [lindex $firstproc 2] } {
	    # ns_schedule_frotz -thread ...
	    ns_start_subprocess detach \
		[list safe_eval [lindex $firstproc 3] \
		     "scheduled procedure" no_value]
	} else {
	    safe_eval [lindex $firstproc 3] "scheduled procedure" no_value
	    aolserver_cleanup
	}

	# Figure out when it'll be run next, and stuff it back in the
	# list.  
	
	set interval [lindex $firstproc 1]

	if {$interval > 0} {
	    set firstproc [lreplace $firstproc 0 0 \
			       [expr $firstproc_time + [lindex $firstproc 1]]]
	    
	    set ns_scheduled_scripts \
		[lreplace $ns_scheduled_scripts 0 0 $firstproc]
	} else {
	    # A -once... delete it.  The sort left it first on the list, so...
	    
	    set ns_scheduled_scripts [lrange $ns_scheduled_scripts 1 end]
	}
    }
}

run_at final_setup {
    ns_start_subprocess die_at_restart {run_scheduled_procs}
}

# Run one-time init scripts

proc run_init_scripts {} {
    global final_init_scripts

    set final_init_scripts [lsort -command sort_by_times $final_init_scripts]

    foreach intvl_script $final_init_scripts {
	aolserver_cleanup
	safe_eval [lindex $intvl_script 1] "scheduled init procedure" no_value
    }
}

# Common ns_schedule_frotz arg parsing.  Using perverse control
# structures to make up for anemic data structures.  What a language...

proc parse_ns_schedule_args { } {

    uplevel {
	global initializing

	if {!$initializing} {
	    error "Can only schedule procedures during server initialization"
	}
	
	set once 0; set thread 0

	while {[string first - [lindex $arglist 0]] == 0 } {
	    set arg [lindex $arglist 0];  set arglist [lrange $arglist 1 end]

	    if ![string compare $arg -once]   { set once 1
	    } elseif ![string compare $arg -thread] { set thread 1
	    } else {
		error "Unrecognized argument $arg"
	    }
	}

	for {set i 0} {$i < [llength $intvl_parms]} {incr i} {
	    set [lindex $intvl_parms $i] [lindex $arglist $i]
	}
	
	set script [lrange $arglist [llength $intvl_parms] end]

	if {[llength $script] == 1} { 
            set script [lindex $script 0] 
        } else {
            set script [concat [list [lindex $script 0]] [lindex $script 1]]
        }
    }
}

# ns_schedule procedures

proc ns_schedule_daily_weekly { arglist weekly } {
    
    # Determine size of interval between invocations, and how far
    # into that interval the thing should run...
    
    if $weekly {
	set intvl_parms {day hr min}; parse_ns_schedule_args
	set secs_into_interval_proc [expr (($day*24 + $hr)*60 + $min)*60]
	set default_interval [expr 7*24*60*60]
    } else {
	set intvl_parms {hr min}; parse_ns_schedule_args 
	set secs_into_interval_proc [expr ($hr*60 + $min)*60]
	set default_interval [expr 24*60*60]
    }
    
    # Determine where we are now in the interval, and when the first
    # run should be triggered.  If the clock ticks between the next
    # two lines, this proc will be off by a second.  We don't care...
    
    set timevec [ns_localtime]
    set time_now [ns_time]
    
    set secs_into_interval_now \
	[expr [lindex $timevec 0] + 60*[lindex $timevec 1] + \
	     60*60*[lindex $timevec 2]]

    if $weekly {
	incr secs_into_interval_now [expr [lindex $timevec 6]*24*60*60]
    }
    
    set interval_start [expr $time_now - $secs_into_interval_now]

    set next_run [expr $interval_start + $secs_into_interval_proc]
    
    # If it's too late to run today/this week, wait till tomorrow/next week
    
    if {$next_run < $time_now} { incr next_run $default_interval }

    # Set interval negative for -once

    if {$once} {set interval -1} {set interval $default_interval}

    # Push it on the list...
    
    global ns_scheduled_scripts
    lappend ns_scheduled_scripts [list $next_run $interval $thread $script]

    return "procedure_ids_not_valid"
}

proc ns_schedule_weekly { args } {
    ns_schedule_daily_weekly $args 1
}

proc ns_schedule_daily { args } {
    ns_schedule_daily_weekly $args 0
}

proc ns_schedule_proc { args } {
    
    global ns_scheduled_scripts final_init_scripts
    
    set arglist $args; set intvl_parms {interval}; parse_ns_schedule_args

    # If we're not initializing, we've been bounced.  If we are
    # initializing, and we're scheduling a -once procedure for less
    # than two minutes in the future, treat it as a special init
    # script, and allow it to run in config context...

    if {$once && $interval < 120} {
	lappend final_init_scripts [list $interval $script]
    } else {
	# Treat as real scheduled procedure...
	set next_run [expr [ns_time] + $interval]
	if $once {set interval -1}
	lappend ns_scheduled_scripts [list $next_run $interval $thread $script]
    }

    return "procedure_ids_not_valid"
}

$slaveInterp alias ns_schedule_proc   ns_schedule_proc
$slaveInterp alias ns_schedule_daily  ns_schedule_daily
$slaveInterp alias ns_schedule_weekly ns_schedule_weekly

