################################################################
#
# Getting information about the server and request.

# Location for the error log.  This is a wild-ass guess unless explicitly
# set (there may be no physical log file if we're using piped logs).

set log_file_location [ap_config_filename log_file_location logs/error_log]

# Various dispatch functions...

proc ns_info {arg} {

    global log_file_location server_name
    
    if {![string compare $arg name]}     { return mod_aolserver }
    if {![string compare $arg version]}  { return 0 }
    if {![string compare $arg winnt]}    { return 0 }
    if {![string compare $arg server]}   { return $server_name }
    if {![string compare $arg label]}    { return "Campbell's soup" }
    if {![string compare $arg platform]} { return Unix }; # ???
    if {![string compare $arg name]}     { return Apache }
    if {![string compare $arg hostname]} { return [ns_info_hostname] }
    if {![string compare $arg log]}      { return $log_file_location }
    
    if {![string compare $arg tcllib]} {
        return [ns_config ns/server/[ns_info server]/tcl Library] 
    }

    if {![string compare $arg pageroot]} {
	return [ns_config ns/server/[ns_info server] pageroot]
    }

    error "Unrecognized ns_info keyword $arg"
}

proc ns_server {arg} {			
    # Need C status interrogation to do this right...
    if {![string compare $arg active]} { return "" }
    error "Unrecognized ns_server keyword $arg"
}

$slaveInterp alias ns_info ns_info
$slaveInterp alias ns_server ns_server

# Getting information about the request (note that some of ns_conn
# is implemented here in tcl).

# Var caching result of ns_conn form... 

lappend per_req_vars ns_conn_form

proc ns_parsequery {query_string} {
    
    set my_set [ns_set create form_args]
    set argpairs [split $query_string "&;"]

    foreach pair $argpairs {
	regsub -all \\+ $pair " " pair
	set pair [split $pair =]
	set var [ns_urldecode [lindex $pair 0]]
	set val [ns_urldecode [lindex $pair 1]]
	ns_set put $my_set $var $val
    }

    return $my_set
}

proc ns_conn {arg args} {

    if {![string compare $arg isconnected]} {
	return [ns_conn_inner isconnected]
    }

    if {![ns_conn_inner isconnected]} {
	error "No connection"
    }
    
    if {![string compare $arg urlv]} {
	# urlv has weird undocumented arg...

	set nargs [llength args]

	set urlv [lrange [split [ns_conn_inner url] /] 1 end]

	switch -- $nargs {
	    2 {
		# ns_conn urlv conn index
		return [lindex $urlv [lindex $args 1]]
	    }
	    1 {
		set index [lindex $args 0]
		if [regexp {^[0-9]+$} $index] {
		    # ns_conn urlv index
		    return [lindex $urlv $index]
		} else {
		    # ns_conn urlv conn
		    return $urlv
		}
	    }
	    0 {
		return $urlv
	    }
	}
    }

    # Rest take one optional argument, the connection id, which we
    # ignore.

    if {[llength args] > 1} {
	error "Bad args --- ns_conn $arg ?conn?"
    }

    # More options implemented in tcl

    if {![string compare $arg urlc]} {
	set urlv [lrange [split [ns_conn_inner url] /] 1 end]
	return [llength $urlv]
    }
    
    if {![string compare $arg form]} {

	global ns_conn_form

	if [info exists ns_conn_form] { return $ns_conn_form }
	
	set ns_conn_form [ns_parsequery [ns_req_data]]
	return $ns_conn_form
	
    }

    if {![string compare $arg location]} {
	# Assume no SSL for now...
	set protocol http
	set port [ns_conn_inner port]
	set servname [ns_conn_inner host]

	if {$port == 80} {set port ""} else { set port ":$port" }
	return "$protocol://$servname$port"
    }

    if {![string compare $arg version]} {
	regexp /(.*) [ns_conn_inner rprotocol] dummy protocol
	return $protocol
    }

    # Let's hope it's something handled by the C code...
    
    ns_conn_inner $arg
}

$slaveInterp alias ns_conn ns_conn
$slaveInterp alias ns_parsequery ns_parsequery

